using System;
using System.Collections;
using Server;
using Server.Mobiles;
using Server.Items;

namespace Server
{
	public class GenerateRares
	{
		public GenerateRares()
		{
		}

		public static void Initialize()
		{
			Server.Commands.Register( "RareGen", AccessLevel.Administrator, new CommandEventHandler( GenerateRares_OnCommand ) );
		}

		[Usage( "RareGen" )]
		[Description( "Generates steable rares in doom." )]
		public static void GenerateRares_OnCommand( CommandEventArgs e )
		{
			e.Mobile.SendMessage( "Please hold generating doom rare system." );

			DecoreGen.CreateDecore();
			LampRoomTeleporter.CreateTeleporters();
			RareSpawner.PlaceDoomSpawners();

			e.Mobile.SendMessage( "Doom rare system generated." );
		}

		public class DecoreGen
		{
			
			public DecoreGen()
			{
			}

			private static Queue m_Queue = new Queue();

			public static bool FindDecore( Map map, Point3D p )
			{
				IPooledEnumerable eable = map.GetItemsInRange( p, 0 );

				foreach ( Item item in eable )
				{
					if ( item is Nightstand )
					{
						int delta = item.Z - p.Z;

						if ( delta >= -12 && delta <= 12 )
							m_Queue.Enqueue( item );
					}
					else if ( item is Pedestal )
					{
						int delta = item.Z - p.Z;

						if ( delta >= -12 && delta <= 12 )
							m_Queue.Enqueue( item );
					}
				}

				eable.Free();

				while ( m_Queue.Count > 0 )
					((Item)m_Queue.Dequeue()).Delete();

				return false;
			}

			public static void CreateDecore( Point3D pointLocation, Map mapLocation )
			{
				Point3D table1 = new Point3D( 445, 9, 0 );
				Point3D table2 = new Point3D( 483, 18, -1 );
				Point3D table3 = new Point3D( 460, 9, 0 );
				Point3D pedestal = new Point3D( 467, 92, -1 );

				if ( !FindDecore( mapLocation, pointLocation ) )
				{
				
					if ( pointLocation ==  table1 )
					{
						Nightstand table = new Nightstand();
						table.Movable = false;
						table.Visible = false;
						table.Name = "DO NOT REMOVE! SPAWN HELPER!";
						table.MoveToWorld( pointLocation, mapLocation );
					}
					else if ( pointLocation ==  table2 )
					{
						Nightstand table = new Nightstand();
						table.Movable = false;
						table.Visible = false;
						table.Name = "DO NOT REMOVE! SPAWN HELPER!";
						table.MoveToWorld( pointLocation, mapLocation );
					}
					else if ( pointLocation ==  table3 )
					{
						Nightstand table = new Nightstand();
						table.Movable = false;
						table.Visible = false;
						table.Name = "DO NOT REMOVE! SPAWN HELPER!";
						table.MoveToWorld( pointLocation, mapLocation );
					}
					else if ( pointLocation ==  pedestal )
					{
						Pedestal ped = new Pedestal();
						ped.Movable = false;
						ped.MoveToWorld( pointLocation, mapLocation );
					}
				}
			}

			public static void CreateDecore( int xLoc, int yLoc, int zLoc, Map map )
			{
				CreateDecore( new Point3D( xLoc, yLoc, zLoc ), map);
			}

			public static void CreateDecoreMalas( Map map )
			{
				CreateDecore( 445, 9, 0, map );
				CreateDecore( 483, 18, -1, map );
				CreateDecore( 460, 9, 0, map );
				CreateDecore( 467, 92, -1, map );
			}


			public static void CreateDecore()
			{
				CreateDecoreMalas( Map.Malas );
			}
		}

		public class LampRoomTeleporter
		{
			
			public LampRoomTeleporter()
			{
			}

			private static Queue m_Queue = new Queue();

			public static bool FindTeleporter( Map map, Point3D p )
			{
				IPooledEnumerable eable = map.GetItemsInRange( p, 0 );

				foreach ( Item item in eable )
				{
					if ( item is Teleporter )
					{
						int delta = item.Z - p.Z;

						if ( delta >= -12 && delta <= 12 )
							m_Queue.Enqueue( item );
					}
				}

				eable.Free();

				while ( m_Queue.Count > 0 )
					((Item)m_Queue.Dequeue()).Delete();

				return false;
			}

			public static void CreateTeleporter( Point3D pointLocation, Map mapLocation )
			{
				Point3D white = new Point3D( 468, 92, -1 );
				Point3D brown = new Point3D( 469, 92, -1 );
				Point3D red = new Point3D( 470, 92, -1 );

				if ( !FindTeleporter( mapLocation, pointLocation ) )
				{
				
					if ( pointLocation ==  white )
					{
						Teleporter port = new Teleporter();
						port.ItemID = 6173;
						port.Visible = true;
						port.Name = "alchemical symbol";
						port.Hue = 1154;
						port.MapDest = Map.Malas;
						port.PointDest = new Point3D( 2354, 1262, -85 );
						port.SoundID = 510;
						port.DestEffect = true;
						port.SourceEffect = true;
						port.MoveToWorld( pointLocation, mapLocation );
					}
					else if ( pointLocation ==  brown )
					{
						Teleporter port = new Teleporter();
						port.ItemID = 6177;
						port.Visible = true;
						port.Name = "alchemical symbol";
						port.Hue = 1839; //TODO This the right hue?
						port.MapDest = Map.Malas;
						port.PointDest = new Point3D( 476, 92, -1 );
						port.SoundID = 510;
						port.DestEffect = true;
						port.SourceEffect = true;
						port.MoveToWorld( pointLocation, mapLocation );
					}
					else if ( pointLocation ==  red )
					{
						Teleporter port = new Teleporter();
						port.ItemID = 6181;
						port.Visible = true;
						port.Name = "alchemical symbol";
						port.Hue = 2118;
						port.MapDest = Map.Malas;
						port.PointDest = new Point3D( 323, 63, -1 );
						port.SoundID = 510;
						port.DestEffect = true;
						port.SourceEffect = true;
						port.MoveToWorld( pointLocation, mapLocation );
					}
				}
			}

			public static void CreateTeleporter( int xLoc, int yLoc, int zLoc, Map map )
			{
				CreateTeleporter( new Point3D( xLoc, yLoc, zLoc ), map);
			}

			public static void CreateTeleportersLampRoom( Map map )
			{
				CreateTeleporter( 468, 92, -1, map );
				CreateTeleporter( 469, 92, -1, map );
				CreateTeleporter( 470, 92, -1, map );
			}


			public static void CreateTeleporters()
			{
				CreateTeleportersLampRoom( Map.Malas );
			}
		}

		public class RareSpawner
		{

			//configuration
			private const bool TotalRespawn = true;//Should we spawn them up right away?
			private static TimeSpan MinTime = TimeSpan.FromMinutes( 75 );//min spawn time
			private static TimeSpan MaxTime = TimeSpan.FromMinutes( 90 );//max spawn time

			public RareSpawner()
			{
			}

			private static Queue m_ToDelete = new Queue();

			public static void ClearSpawners( int x, int y, int z, Map map )
			{
				IPooledEnumerable eable = map.GetItemsInRange( new Point3D( x, y, z ), 0 );

				foreach ( Item item in eable )
				{
					if ( item is DoomSpawner && item.Z == z )
						m_ToDelete.Enqueue( item );
				}

				eable.Free();

				while ( m_ToDelete.Count > 0 )
					((Item)m_ToDelete.Dequeue()).Delete();
			}

			public static void PlaceDoomSpawners()
			{
				//Rares
				PlaceRareSpawns( 312, 55, -1, "RockRare" );
				PlaceRareSpawns( 347, 41, -1, "SkinnedGoatRare" );
				PlaceRareSpawns( 352, 32, -1, "SkinnedDeerRare" );
				PlaceRareSpawns( 361, 31, 5, "SkullCandleRare" );
				PlaceRareSpawns( 422, 25, 0, "EggCaseRare" );
				PlaceRareSpawns( 423, 23, -1, "CocoonRare" );
				PlaceRareSpawns( 432, 19, -1, "StretchedHideRare" );
				PlaceRareSpawns( 432, 12, -1, "SaddleRare" );
				PlaceRareSpawns( 462, 10, -1, "BrazierRare" );
				PlaceRareSpawns( 498, 54, -1, "GruesomeStandardRare" );
				PlaceRareSpawns( 471, 92, -1, "LampPostRare" );
				PlaceRareSpawns( 429, 193, -1, "FallenBookRare" );
				PlaceRareSpawns( 426, 189, -1, "StackedBooks2Rare" );
				PlaceRareSpawns( 418, 198, 0, "StackedBooksRare" );
				PlaceRareSpawns( 460, 9, 6, "LeatherTunicRare" );
				PlaceRareSpawns( 491, 8, 5, "StuddedTunic2Rare" );
				PlaceRareSpawns( 483, 18, 5, "StuddedTunicRare" );
				PlaceRareSpawns( 445, 9, 6, "StuddedLeggingsRare" );
				PlaceRareSpawns( 381, 375, 10, "BloodyWaterRare" );
				PlaceRareSpawns( 378, 372, 0, "DamagedBooksRare" );
				PlaceRareSpawns( 369, 371, 0, "BottleRare" );
				PlaceRareSpawns( 489, 370, 5, "TarotCardsRare" );
				PlaceRareSpawns( 497, 368, 12, "ReverseBackpackRare" );
				PlaceRareSpawns( 488, 364, -1, "RuinedPaintingRare" );

				//Artifacts
				PlaceRareSpawns( 354, 176, 8, "TitansHammerRare" );
				PlaceRareSpawns( 467, 92, 4, "InquisitorsResolutionRare" );
				PlaceRareSpawns( 259, 72, -1, "ZyronicClawRare" );
				PlaceRareSpawns( 369, 389, -1, "BladeOfTheRighteousRare" );
			}

			public static void PlaceRareSpawns( int x, int y, int z, string types )
			{

				switch ( types )
				{
					case "RockRare":
						MakeDoomSpawner( "RockRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromMinutes( 75 );
						MaxTime = TimeSpan.FromMinutes( 90 );
						break;
					case "SkinnedGoatRare":
						MakeDoomSpawner( "SkinnedGoatRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "SkinnedDeerRare":
						MakeDoomSpawner( "SkinnedDeerRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 96 );
						MaxTime = TimeSpan.FromHours( 96 );
						break;
					case "SkullCandleRare":
						MakeDoomSpawner( "SkullCandleRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromMinutes( 75 );
						MaxTime = TimeSpan.FromMinutes( 90 );
						break;
					case "EggCaseRare":
						MakeDoomSpawner( "EggCaseRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "CocoonRare":
						MakeDoomSpawner( "CocoonRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 96 );
						MaxTime = TimeSpan.FromHours( 96 );
						break;
					case "StretchedHideRare":
						MakeDoomSpawner( "StretchedHideRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 3 );
						MaxTime = TimeSpan.FromHours( 6 );
						break;
					case "SaddleRare":
						MakeDoomSpawner( "SaddleRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 384 );
						MaxTime = TimeSpan.FromHours( 386 );
						break;
					case "BrazierRare":
						MakeDoomSpawner( "BrazierRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 3 );
						MaxTime = TimeSpan.FromHours( 6 );
						break;
					case "GruesomeStandardRare":
						MakeDoomSpawner( "GruesomeStandardRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "LampPostRare":
						MakeDoomSpawner( "LampPostRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 6 );
						MaxTime = TimeSpan.FromHours( 8 );
						break;
					case "FallenBookRare":
						MakeDoomSpawner( "FallenBookRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 6 );
						MaxTime = TimeSpan.FromHours( 8 );
						break;
					case "StackedBooks2Rare":
						MakeDoomSpawner( "StackedBooks2Rare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 6 );
						MaxTime = TimeSpan.FromHours( 8 );
						break;
					case "StackedBooksRare":
						MakeDoomSpawner( "StackedBooksRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 6 );
						MaxTime = TimeSpan.FromHours( 8 );
						break;
					case "LeatherTunicRare":
						MakeDoomSpawner( "LeatherTunicRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 384 );
						MaxTime = TimeSpan.FromHours( 384 );
						break;
					case "StuddedTunicRare":
						MakeDoomSpawner( "StuddedTunic2Rare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 96 );
						MaxTime = TimeSpan.FromHours( 96 );
						break;
					case "StuddedTunic2Rare":
						MakeDoomSpawner( "StuddedTunicRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 96 );
						MaxTime = TimeSpan.FromHours( 96 );
						break;
					case "StuddedLeggingsRare":
						MakeDoomSpawner( "StuddedLeggingsRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "BloodyWaterRare":
						MakeDoomSpawner( "BloodyWaterRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "DamagedBooksRare":
						MakeDoomSpawner( "DamagedBooksRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromMinutes( 75 );
						MaxTime = TimeSpan.FromMinutes( 90 );
						break;
					case "BottleRare":
						MakeDoomSpawner( "BottleRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromMinutes( 75 );
						MaxTime = TimeSpan.FromMinutes( 90 );
						break;
					case "TarotCardsRare":
						MakeDoomSpawner( "TarotCardsRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "ReverseBackpackRare":
						MakeDoomSpawner( "ReverseBackpackRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 24 );
						MaxTime = TimeSpan.FromHours( 32 );
						break;
					case "RuinedPaintingRare":
						MakeDoomSpawner( "RuinedPaintingRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 720 );
						MaxTime = TimeSpan.FromHours( 720 );
						break;
					case "TitansHammerRare":
						MakeDoomSpawner( "TitansHammerRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 720 );
						MaxTime = TimeSpan.FromHours( 720 );
						break;
					case "InquisitorsResolutionRare":
						MakeDoomSpawner( "InquisitorsResolutionRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 720 );
						MaxTime = TimeSpan.FromHours( 720 );
						break;
					case "ZyronicClawRare":
						MakeDoomSpawner( "ZyronicClawRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 720 );
						MaxTime = TimeSpan.FromHours( 720 );
						break;
					case "BladeOfTheRighteousRare":
						MakeDoomSpawner( "BladeOfTheRighteousRare", x, y, z, Map.Malas, true );
						MinTime = TimeSpan.FromHours( 720 );
						MaxTime = TimeSpan.FromHours( 720 );
						break;
					default:
						break;
				}
			}

			private static void MakeDoomSpawner( string types, int x, int y, int z, Map map, bool start )
			{
				ClearSpawners( x, y, z, map );

				DoomSpawner sp = new DoomSpawner( types );

				sp.Count = 1;

				sp.Running = true;
				sp.HomeRange = 0;
				sp.MinDelay = MinTime;
				sp.MaxDelay = MaxTime;

				sp.MoveToWorld( new Point3D( x, y, z ), map );

			}
		}
	}
}
